#!/bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/../../../modules/utils.sh"

export OVERLAY_RUNTIME_IMAGE="registry.uipath.com/uipath/sf-k8-utils-rhel:22.10.10"
function set_runtime_image_offline(){
    [[ -z "$(kubectl -n docker-registry get pod -l=app=docker-registry -o name)" ]] && return
    if [[ $(kubectl -n docker-registry get deploy docker-registry -o jsonpath="{.status.readyReplicas}") != "1" ]];
    then
        warn "Docker registry is not up"
        return 
    fi
    export OVERLAY_RUNTIME_IMAGE="localhost:30071/uipath/sf-k8-utils-rhel:22.10.10"
}

function delete_daemonset(){
    kubectl delete ds overlaytest &>/dev/null
    until [ "$(kubectl get pods -l name=overlaytest 2>/dev/null | grep -c overlay)" == "0" ]  ; do sleep 5; done
}

function create_overlay_daemon(){
    delete_daemonset
    set_runtime_image_offline
    cat << EOF | kubectl create -f - 1>/dev/null
apiVersion: apps/v1
kind: DaemonSet
metadata:
  name: overlaytest
spec:
  selector:
      matchLabels:
        name: overlaytest
  template:
    metadata:
      labels:
        name: overlaytest
        app.kubernetes.io/component: test
    spec:
      tolerations:
      - operator: Exists
      containers:
      - image: ${OVERLAY_RUNTIME_IMAGE}
        imagePullPolicy: Always
        name: overlaytest
        command: ["sh", "-c", "tail -f /dev/null"]
        terminationMessagePath: /dev/termination-log
EOF
    kubectl rollout status ds/overlaytest 1>/dev/null
}

function connectivity_test(){
    section "Network Connectivity Test"
    kubectl get pods --field-selector=status.phase=Running -l name=overlaytest -o jsonpath='{range .items[*]}{@.metadata.name}{" "}{@.spec.nodeName}{"\n"}{end}' |
    while read -r spod shost 
        do kubectl get pods --field-selector=status.phase=Running -l name=overlaytest -o jsonpath='{range .items[*]}{@.status.podIP}{" "}{@.spec.nodeName}{"\n"}{end}' |
        while read -r tip thost
        do kubectl --request-timeout='10s' exec "$spod" -c overlaytest -- /bin/sh -c "ping -c2 $tip > /dev/null 2>&1"
            RC=$?
            if [ $RC -ne 0 ]
            then add_to_report error "FAIL: $spod on $shost cannot reach pod IP $tip on $thost"
            fi
        done
    done
}

function bandwidth_test(){
    kubectl get pods --field-selector=status.phase=Running -l name=overlaytest -o jsonpath='{range .items[*]}{@.metadata.name}{" "}{@.status.podIP}{" "}{@.spec.nodeName}{"\n"}{end}' |
    while read -r spod sip shost 
        do 
        section "Network Bandwidth Test to $shost"
        kubectl exec "$spod" -c overlaytest -- /bin/sh -c "nohup iperf -s >> /tmp/listen.txt 2>&1 &"
        kubectl get pods --field-selector=status.phase=Running -l name=overlaytest -o jsonpath='{range .items[*]}{@.metadata.name}{" "}{@.spec.nodeName}{"\n"}{end}' |
        while read -r cpod chost
        do
            [[ "$chost" == "$shost" ]] && continue
            add_to_report echo "Node $chost ===========> $shost"
            response=$(kubectl exec "$cpod" -c overlaytest -- /bin/sh -c "iperf -c $sip -f MBytes -x CMS")
            add_to_report echo "$response"
        done
    done
}

function main(){
    create_overlay_daemon
    connectivity_test
    bandwidth_test
    delete_daemonset
}

main