import datetime as dt
import json
import os
import sys

from azure.identity import ManagedIdentityCredential
from azure.storage.blob import (
    BlobClient,
    BlobSasPermissions,
    BlobServiceClient,
    generate_blob_sas,
)

storage_acct_name = sys.argv[1]
container_name = sys.argv[2]
blob_name = sys.argv[3]
user_client_id = sys.argv[4]
url_suffix = sys.argv[5]

credential = ManagedIdentityCredential()
if user_client_id != "":
  credential=ManagedIdentityCredential(client_id=user_client_id)

url = f"https://{storage_acct_name}{url_suffix}"
blob_service_client = BlobServiceClient(url, credential=credential)
udk = blob_service_client.get_user_delegation_key(
    key_start_time=dt.datetime.utcnow() - dt.timedelta(hours=1),
    key_expiry_time=dt.datetime.utcnow() + dt.timedelta(hours=1))

sas = generate_blob_sas(
    account_name=storage_acct_name,
    container_name=container_name,
    blob_name=blob_name,
    user_delegation_key=udk,
    permission=BlobSasPermissions(read=True),
    start = dt.datetime.utcnow() - dt.timedelta(minutes=15),
    expiry = dt.datetime.utcnow() + dt.timedelta(hours=2),
)

sas_url = (
    f'https://{storage_acct_name}{url_suffix}/'
    f'{container_name}/{blob_name}?{sas}'
)

print('SupportBundleLink '+sas_url)

